# mpcaHydro - Modules for downloading hydrology data from MPCA servers and databases

# Modules are imported lazily to avoid issues with missing data files
# Use: from mpcaHydro import calibration_config




from mpcaHydro import data_manager_functions as dmf
from mpcaHydro import wiski

station_ids = ['E67013001']


dmf.download_wiski_data(con,station_ids,start_year = 2010,)



station_ids = ['H67009001',
 'H67012001',
 'H67013001',
 'H67014001',
 'H67016001',
 'H67014002',
 'E67013001',
 'W67036001',
 'W67004001']


# -*- coding: utf-8 -*-
"""Integration tests for EQuIS data operations.

Note: These tests require Oracle database credentials and network access.
Most tests use mocks to avoid requiring actual credentials.
"""

import tempfile
from pathlib import Path
from unittest.mock import patch, MagicMock
import pytest
import pandas as pd


# Check if warehouse module can be imported (requires data files)
try:
    from mpcaHydro import warehouse
    WAREHOUSE_AVAILABLE = True
except Exception:
    WAREHOUSE_AVAILABLE = False
    warehouse = None

requires_warehouse = pytest.mark.skipif(
    not WAREHOUSE_AVAILABLE,
    reason="Warehouse module not available (missing data files)"
)


class TestEquisConnectionMocked:
    """Tests for EQuIS connection with mocks."""

    def test_connect_function_exists(self):
        """Test that equis.connect function exists and is callable."""
        from mpcaHydro import equis
        
        assert callable(equis.connect)
        assert callable(equis.close_connection)

    def test_connect_accepts_connection_parameter(self):
        """Test that download accepts optional connection parameter."""
        import inspect
        from mpcaHydro import equis
        
        sig = inspect.signature(equis.download)
        assert 'connection' in sig.parameters

    def test_close_connection_accepts_connection_parameter(self):
        """Test that close_connection accepts optional connection parameter."""
        import inspect
        from mpcaHydro import equis
        
        sig = inspect.signature(equis.close_connection)
        assert 'connection' in sig.parameters


class TestEquisTransform:
    """Tests for EQuIS data transformation."""

    def test_transform_exists(self):
        """Test that equis.transform function exists."""
        from mpcaHydro import equis
        
        assert callable(equis.transform)

    def test_transform_with_mock_data(self):
        """Test EQuIS transform with mock data."""
        from mpcaHydro import equis
        from datetime import datetime
        
        # Create mock EQuIS data structure
        mock_data = pd.DataFrame({
            'SYS_LOC_CODE': ['S-001', 'S-001'],
            'CAS_RN': ['SOLIDS-TSS', 'SOLIDS-TSS'],
            'SAMPLE_DATE_TIME': [datetime(2020, 1, 1, 10, 0), datetime(2020, 1, 1, 11, 0)],
            'SAMPLE_DATE_TIMEZONE': ['CST', 'CST'],
            'RESULT_NUMERIC': [50.0, 55.0],
            'RESULT_UNIT': ['mg/L', 'mg/L'],
        })
        
        # Transform should handle this data
        try:
            result = equis.transform(mock_data)
            assert isinstance(result, pd.DataFrame)
        except Exception as e:
            # May fail due to missing columns in mock, which is expected
            pass

    def test_cas_rn_mapping(self):
        """Test that CAS_RN mapping dict is defined."""
        from mpcaHydro import equis
        
        assert hasattr(equis, 'CAS_RN_MAP')
        assert isinstance(equis.CAS_RN_MAP, dict)
        assert 'SOLIDS-TSS' in equis.CAS_RN_MAP
        assert equis.CAS_RN_MAP['SOLIDS-TSS'] == 'TSS'


class TestEquisDownloadPlaceholder:
    """Placeholder tests for EQuIS download (requires credentials)."""

    @pytest.mark.integration
    @pytest.mark.network
    @pytest.mark.skip(reason="Requires Oracle credentials")
    def test_download_equis_data(self):
        """Test downloading EQuIS data (requires Oracle credentials)."""
        from mpcaHydro import equis
        import os
        
        username = os.environ.get('ORACLE_USER')
        password = os.environ.get('ORACLE_PASSWORD')
        
        if not username or not password:
            pytest.skip("Oracle credentials not available")
        
        conn = equis.connect(user=username, password=password)
        
        station_ids = ['S-001234']  # Example station
        df = equis.download(station_ids, connection=conn)
        
        equis.close_connection(conn)
        
        assert isinstance(df, pd.DataFrame)


class TestEquisWithMockedDownload:
    """Tests for EQuIS operations using mocked Oracle connection."""

    @requires_warehouse
    def test_download_equis_data_mocked(self):
        """Test download_equis_data with mocked Oracle."""
        from mpcaHydro import data_manager_functions as dmf
        from mpcaHydro import warehouse
        from mpcaHydro import equis
        from datetime import datetime
        
        # Create mock EQuIS raw data
        mock_raw_data = pd.DataFrame({
            'LATITUDE': [44.5, 44.5],
            'LONGITUDE': [-93.2, -93.2],
            'WID_LIST': ['123', '123'],
            'SAMPLE_METHOD': ['G', 'G'],
            'SAMPLE_REMARK': [None, None],
            'SYS_LOC_CODE': ['S-001', 'S-001'],
            'CAS_RN': ['SOLIDS-TSS', '7723-14-0'],
            'SAMPLE_DATE_TIME': [datetime(2020, 1, 1, 10, 0), datetime(2020, 1, 1, 10, 0)],
            'SAMPLE_DATE_TIMEZONE': ['CST', 'CST'],
            'RESULT_NUMERIC': [50.0, 0.5],
            'RESULT_UNIT': ['mg/L', 'mg/L'],
        })
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            with warehouse.connect(db_path.as_posix(), read_only=False) as con:
                # Load mock staging data
                warehouse.load_df_to_table(con, mock_raw_data, 'staging.equis')
                
                # Verify data was loaded
                count = con.execute("SELECT COUNT(*) FROM staging.equis").fetchone()[0]
                assert count == 2


if __name__ == '__main__':
    pytest.main([__file__, '-v'])



# -*- coding: utf-8 -*-
"""Integration tests for WISKI data operations.

Note: Network tests require access to the WISKI web service.
"""

import tempfile
from pathlib import Path
from unittest.mock import patch, MagicMock
import pytest
import pandas as pd


# Check if warehouse module can be imported (requires data files)
try:
    from mpcaHydro import warehouse
    WAREHOUSE_AVAILABLE = True
except Exception:
    WAREHOUSE_AVAILABLE = False
    warehouse = None

requires_warehouse = pytest.mark.skipif(
    not WAREHOUSE_AVAILABLE,
    reason="Warehouse module not available (missing data files)"
)


# Check if WISKI service is accessible
try:
    from mpcaHydro import pywisk
    is_up, _ = pywisk.test_connection()
    NETWORK_AVAILABLE = is_up
except Exception:
    NETWORK_AVAILABLE = False

skip_network = pytest.mark.skipif(
    not NETWORK_AVAILABLE,
    reason="WISKI network service not available"
)


class TestPyWiskConnection:
    """Tests for pywisk service connection."""

    @skip_network
    @pytest.mark.integration
    @pytest.mark.network
    def test_wiski_service_connection(self):
        """Test connection to WISKI web service (requires network)."""
        from mpcaHydro import pywisk
        
        is_up, message = pywisk.test_connection()
        # Note: This test may fail if the WISKI service is down
        # It's marked as integration/network test
        print(f"WISKI service status: {message}")


class TestWiskiDownload:
    """Tests for downloading WISKI data."""

    @skip_network
    @pytest.mark.integration
    @pytest.mark.network
    def test_download_station_data(self):
        """Test downloading data for a single station (requires network)."""
        from mpcaHydro import pywisk
        
        # Use a known station ID
        station_ids = ['H67014001']
        
        # Get timeseries IDs for the station
        df_ts = pywisk.get_ts_ids(station_nos=station_ids)
        
        assert not df_ts.empty
        assert 'ts_id' in df_ts.columns
        assert 'station_no' in df_ts.columns

    @skip_network
    @pytest.mark.integration
    @pytest.mark.network
    def test_get_stations_returns_dataframe(self):
        """Test that get_stations returns a DataFrame (requires network)."""
        from mpcaHydro import pywisk
        
        df = pywisk.get_stations(station_no=['H67014001'])
        
        assert isinstance(df, pd.DataFrame)
        assert 'station_no' in df.columns

    @skip_network
    @pytest.mark.integration
    @pytest.mark.network
    def test_get_ts_returns_data(self):
        """Test that get_ts returns timeseries data (requires network)."""
        from mpcaHydro import pywisk
        
        # First get a valid ts_id
        station_ids = ['H67014001']
        df_ts = pywisk.get_ts_ids(station_nos=station_ids, parametertype_id='11500')  # Flow
        
        if not df_ts.empty:
            ts_id = df_ts['ts_id'].iloc[0]
            df = pywisk.get_ts(ts_id, start_date='2020-01-01', end_date='2020-01-31')
            
            assert isinstance(df, pd.DataFrame)
            if not df.empty:
                assert 'Timestamp' in df.columns
                assert 'Value' in df.columns


class TestWiskiDataWithMock:
    """Tests for WISKI data operations using mocks."""

    @requires_warehouse
    def test_download_with_mock_data(self):
        """Test WISKI download flow with mocked service."""
        from mpcaHydro import data_manager_functions as dmf
        from mpcaHydro import warehouse
        
        # Create mock data that mimics WISKI download format
        mock_wiski_data = pd.DataFrame({
            'station_no': ['H67014001', 'H67014001'],
            'Timestamp': ['2020-01-01 00:00', '2020-01-01 01:00'],
            'Value': [100.0, 105.0],
            'Quality Code': [1, 1],
            'ts_unitsymbol': ['cfs', 'cfs'],
            'parametertype_id': ['11500', '11500'],
            'parametertype_name': ['Discharge', 'Discharge'],
        })
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            with warehouse.connect(db_path.as_posix(), read_only=False) as con:
                # Load mock data directly
                warehouse.load_df_to_table(con, mock_wiski_data, 'staging.wiski')
                
                # Verify data loaded
                result = con.execute("SELECT COUNT(*) FROM staging.wiski").fetchone()[0]
                assert result == 2


class TestWiskiTransform:
    """Tests for WISKI data transformation."""

    def test_transform_creates_analytics_data(self):
        """Test that WISKI transform creates properly formatted analytics data."""
        # This would test the wiski.transform() function
        # Currently skipping as it requires specific data format
        pass


if __name__ == '__main__':
    pytest.main([__file__, '-v'])



from typing import List

import duckdb
import pandas as pd
from pathlib import Path
from mpcaHydro import outlets
from mpcaHydro import sql_loader

def init_db(db_path: str,reset: bool = False):
    """Initialize the DuckDB database: create schemas and tables."""
    db_path = Path(db_path)
    if reset and db_path.exists():
        db_path.unlink()

    with connect(db_path.as_posix()) as con:
        # Create all schemas
        create_schemas(con)

        # Create tables
        create_outlets_tables(con)
        create_mapping_tables(con)
        create_staging_tables(con)
        create_analytics_tables(con)
        

        # Create views
        update_views(con)



def create_schemas(con: duckdb.DuckDBPyConnection):
    """Create staging, analytics, hspf, and reports schemas if they do not exist."""
    con.execute(sql_loader.get_schemas_sql())

def create_staging_tables(con: duckdb.DuckDBPyConnection):
    """Create necessary tables in the staging schema."""
    con.execute(sql_loader.get_staging_tables_sql())


def create_analytics_tables(con: duckdb.DuckDBPyConnection):
    """Create necessary tables in the analytics schema."""
    con.execute(sql_loader.get_analytics_tables_sql())

def create_mapping_tables(con: duckdb.DuckDBPyConnection):
    """Create and populate tables in the mappings schema from Python dicts and CSVs."""
    # WISKI parametertype_id -> constituent
    wiski_parametertype_map = {
        '11522': 'TP', 
        '11531': 'TP', 
        '11532': 'TSS', 
        '11523': 'TSS',
        '11526': 'N', 
        '11519': 'N', 
        '11520': 'OP', 
        '11528': 'OP',
        '11530': 'TKN', 
        '11521': 'TKN', 
        '11500': 'Q', 
        '11504': 'WT',
        '11533': 'DO', 
        '11507': 'WL'
    }
    df_wiski_params = pd.DataFrame(wiski_parametertype_map.items(), columns=['parametertype_id', 'constituent'])
    con.execute("CREATE TABLE IF NOT EXISTS mappings.wiski_parametertype AS SELECT * FROM df_wiski_params")

    # EQuIS cas_rn -> constituent
    equis_casrn_map = {
        '479-61-8': 'CHLA', 
        'CHLA-CORR': 'CHLA', 
        'BOD': 'BOD', 
        'NO2NO3': 'N',
        '14797-55-8': 'NO3', 
        '14797-65-0': 'NO2', 
        '14265-44-2': 'OP',
        'N-KJEL': 'TKN', 
        'PHOSPHATE-P': 'TP', 
        '7723-14-0': 'TP',
        'SOLIDS-TSS': 'TSS', 
        'TEMP-W': 'WT', 
        '7664-41-7': 'NH3'
    }
    df_equis_cas = pd.DataFrame(equis_casrn_map.items(), columns=['cas_rn', 'constituent'])
    con.execute("CREATE TABLE IF NOT EXISTS mappings.equis_casrn AS SELECT * FROM df_equis_cas")

    # Load station cross-reference from CSV
    xref_csv_path = Path(__file__).parent / 'data/WISKI_EQUIS_XREF.csv'
    if xref_csv_path.exists():
        con.execute(f"CREATE TABLE IF NOT EXISTS mappings.station_xref AS SELECT * FROM read_csv_auto('{xref_csv_path.as_posix()}')")
    else:
        print(f"Warning: WISKI_EQUIS_XREF.csv not found at {xref_csv_path}")

    # Load wiski_quality_codes from CSV
    wiski_qc_csv_path = Path(__file__).parent / 'data/WISKI_QUALITY_CODES.csv'
    if wiski_qc_csv_path.exists():
        con.execute(f"CREATE TABLE IF NOT EXISTS mappings.wiski_quality_codes AS SELECT * FROM read_csv_auto('{wiski_qc_csv_path.as_posix()}')")
    else:
        print(f"Warning: WISKI_QUALITY_CODES.csv not found at {wiski_qc_csv_path}")

def create_outlets_tables(con: duckdb.DuckDBPyConnection):
    """Create tables in the outlets schema to define outlet-station-reach relationships."""
    con.execute(sql_loader.get_outlets_schema_sql())
    con.execute(sql_loader.get_views_outlets_sql())
    outlets.build_outlets(con)


def create_filtered_wiski_view(con: duckdb.DuckDBPyConnection, data_codes: list):
    """Create a view filtering WISKI data based on specified data codes."""
    placeholders = ', '.join(['?'] * len(data_codes))
    query = f"""
    CREATE OR REPLACE VIEW analytics.wiski_filtered AS
    SELECT *
    FROM analytics.wiski_normalized
    WHERE quality_code IN ({placeholders});
    """
    con.execute(query, data_codes)


def create_aggregated_wiski_view(con: duckdb.DuckDBPyConnection):
    """Create a view aggregating WISKI data by hour, station, and constituent."""
    con.execute("""
    CREATE OR REPLACE Table analytics.wiski_aggregated AS
    SELECT 
        station_id,
        constituent,
        time_bucket(INTERVAL '1 hour', datetime) AS hour_start,
        AVG(value) AS value,
        unit
    FROM analytics.wiski_normalized
    GROUP BY 
        station_id, 
        constituent, 
        hour_start,
        unit;
    """)


def update_views(con: duckdb.DuckDBPyConnection):
    """Update all views in the database by loading from SQL files."""
    con.execute(sql_loader.get_views_analytics_sql())
    con.execute(sql_loader.get_views_reports_sql())
    
def connect(db_path: str, read_only: bool = False) -> duckdb.DuckDBPyConnection:
    """
    Returns a DuckDB connection to the given database path.
    Ensures the parent directory exists.
    """
    db_path = Path(db_path)
    parent = db_path.parent
    parent.mkdir(parents=True, exist_ok=True)
    return duckdb.connect(database=db_path.as_posix(), read_only=read_only)


def drop_station_data(con: duckdb.DuckDBPyConnection, station_ids: List[str], station_origin: str):
    """
    Drop all data for a specific stations from staging and analytics schemas.
    """
    placeholders = ', '.join(['?'] * len(station_ids))
    con.execute(f"DELETE FROM staging.equis WHERE station_id IN ({placeholders}) AND station_origin = ?", station_ids + [station_origin])
    con.execute(f"DELETE FROM staging.wiski WHERE station_id IN ({placeholders}) AND station_origin = ?", station_ids + [station_origin])
    con.execute(f"DELETE FROM analytics.equis WHERE station_id IN ({placeholders}) AND station_origin = ?", station_ids + [station_origin])
    con.execute(f"DELETE FROM analytics.wiski WHERE station_id IN ({placeholders}) AND station_origin = ?", station_ids + [station_origin])
    update_views(con)

def get_column_names(con: duckdb.DuckDBPyConnection, table_schema: str, table_name: str) -> list:
    """
    Get the column names of a DuckDB table.
    """
    #table_schema, table_name = table_name.split('.')
    query = """
    SELECT column_name
    FROM information_schema.columns
    WHERE table_name = ? AND table_schema = ?
    """
    result = con.execute(query,[table_name,table_schema]).fetchall()
    column_names = [row[0] for row in result]
    return column_names


def add_df_to_table(con: duckdb.DuckDBPyConnection, df: pd.DataFrame, table_schema: str, table_name: str):
    """
    Append a pandas DataFrame into a DuckDB table. This will create the table
    if it does not exist.
    """
    # get existing columns
    existing_columns = get_column_names(con, table_schema, table_name)
    df = df[existing_columns]


    # register pandas DF and create table if not exists
    con.register("tmp_df", df)

    con.execute(f"""
        INSERT INTO {table_schema}.{table_name} 
        SELECT * FROM tmp_df
    """)
    con.unregister("tmp_df")

def load_df_to_table(con: duckdb.DuckDBPyConnection, df: pd.DataFrame, table_name: str):
    """
    Persist a pandas DataFrame into a DuckDB table. This will overwrite the table
    by default (replace=True).
    """
    # register pandas DF and create table
    con.register("tmp_df", df)
    con.execute(f"CREATE OR REPLACE TABLE {table_name} AS SELECT * FROM tmp_df")
    con.unregister("tmp_df")

def load_df_to_staging(con: duckdb.DuckDBPyConnection, df: pd.DataFrame, table_name: str, replace: bool = True):
    """
    Persist a pandas DataFrame into a staging table. This will overwrite the staging
    table by default (replace=True).
    """
    if replace:
        con.execute(f"DROP TABLE IF EXISTS staging.{table_name}")
    # register pandas DF and create table
    con.register("tmp_df", df)
    con.execute(f"CREATE TABLE staging.{table_name} AS SELECT * FROM tmp_df")
    con.unregister("tmp_df")

def load_csv_to_staging(con: duckdb.DuckDBPyConnection, csv_path: str, table_name: str, replace: bool = True, **read_csv_kwargs):
    """
    Persist a CSV file into a staging table. This will overwrite the staging
    table by default (replace=True).
    """
    if replace:
        con.execute(f"DROP TABLE IF EXISTS staging.{table_name}")
    con.execute(f"""
        CREATE TABLE staging.{table_name} AS 
        SELECT * FROM read_csv_auto('{csv_path}', {', '.join(f"{k}={repr(v)}" for k, v in read_csv_kwargs.items())})
    """)
 
def load_parquet_to_staging(con: duckdb.DuckDBPyConnection, parquet_path: str, table_name: str, replace: bool = True):
    """
    Persist a Parquet file into a staging table. This will overwrite the staging
    table by default (replace=True).
    """
    if replace:
        con.execute(f"DROP TABLE IF EXISTS staging.{table_name}")
    con.execute(f"""
        CREATE TABLE staging.{table_name} AS 
        SELECT * FROM read_parquet('{parquet_path}')
    """)


def write_table_to_parquet(con: duckdb.DuckDBPyConnection, table_name: str, path: str, compression="snappy"):
    """
    Persist a DuckDB table into a Parquet file.
    """
    con.execute(f"COPY (SELECT * FROM {table_name}) TO '{path}' (FORMAT PARQUET, COMPRESSION '{compression}')")


def write_table_to_csv(con: duckdb.DuckDBPyConnection, table_name: str, path: str, header: bool = True, sep: str = ',', **kwargs):
    """
    Persist a DuckDB table into a CSV file.
    """
    con.execute(f"COPY (SELECT * FROM {table_name}) TO '{path}' (FORMAT CSV, HEADER {str(header).upper()}, DELIMITER '{sep}' {', '.join(f', {k}={repr(v)}' for k, v in kwargs.items())})")




def load_df_to_analytics(con: duckdb.DuckDBPyConnection, df: pd.DataFrame, table_name: str):
    """
    Persist a pandas DataFrame into an analytics table.
    """
    con.execute(f"DROP TABLE IF EXISTS analytics.{table_name}")
    con.register("tmp_df", df)
    con.execute(f"CREATE TABLE analytics.{table_name} AS SELECT * FROM tmp_df")
    con.unregister("tmp_df")


def migrate_staging_to_analytics(con: duckdb.DuckDBPyConnection, staging_table: str, analytics_table: str):
    """
    Migrate data from a staging table to an analytics table.
    """
    con.execute(f"DROP TABLE IF EXISTS analytics.{analytics_table}")
    con.execute(f"""
        CREATE TABLE analytics.{analytics_table} AS 
        SELECT * FROM staging.{staging_table}
    """)


def load_to_analytics(con: duckdb.DuckDBPyConnection, table_name: str):
    con.execute(f"""
                CREATE OR REPLACE TABLE analytics.{table_name} AS
                SELECT
                station_id,
                constituent,
                datetime,
                value AS observed_value,
                time_bucket(INTERVAL '1 hour', datetime) AS hour_start,
                AVG(observed_value) AS value
                FROM
                    staging.equis_processed
                GROUP BY
                    hour_start,
                    constituent,
                    station_id
                ORDER BY
                    station_id,
                    constituent
                """)
    # register pandas DF and create table
    con.register("tmp_df", df)
    con.execute(f"CREATE TABLE analytics.{table_name} AS SELECT * FROM tmp_df")
    con.unregister("tmp_df")

def dataframe_to_parquet(con: duckdb.DuckDBPyConnection,  df: pd.DataFrame, path, compression="snappy"):
    # path should be a filename like 'data/raw/equis/equis-20251118.parquet'
    con = duckdb.connect()
    con.register("tmp_df", df)
    con.execute(f"COPY (SELECT * FROM tmp_df) TO '{path}' (FORMAT PARQUET, COMPRESSION '{compression}')")
    con.unregister("tmp_df")
    con.close()


# -*- coding: utf-8 -*-
"""Integration tests for data manager functions.

Note: Many tests require data files in src/mpcaHydro/data/ which may not be present.
These tests will be skipped if required files are missing.
"""

import tempfile
from pathlib import Path
from unittest.mock import patch, MagicMock
import pytest
import pandas as pd


# Check if warehouse module can be imported (requires data files)
try:
    from mpcaHydro import warehouse
    WAREHOUSE_AVAILABLE = True
except Exception:
    WAREHOUSE_AVAILABLE = False
    warehouse = None

requires_warehouse = pytest.mark.skipif(
    not WAREHOUSE_AVAILABLE,
    reason="Warehouse module not available (missing data files)"
)


class TestDataManagerFunctionsIntegration:
    """Integration tests for data_manager_functions module."""

    def test_get_db_path(self):
        """Test get_db_path returns correct path."""
        from mpcaHydro.data_manager_functions import get_db_path
        
        result = get_db_path('/my/folder')
        assert result == Path('/my/folder/observations.duckdb')

    @requires_warehouse
    def test_init_warehouse_integration(self):
        """Test init_warehouse creates and returns path."""
        from mpcaHydro.data_manager_functions import init_warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            result = init_warehouse(db_path, reset=True)
            
            assert result == db_path
            assert db_path.exists()

    @requires_warehouse
    def test_update_views_integration(self):
        """Test update_views works with initialized warehouse."""
        from mpcaHydro.data_manager_functions import init_warehouse, update_views
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            init_warehouse(db_path, reset=True)
            
            with warehouse.connect(db_path.as_posix(), read_only=False) as con:
                # Should not raise
                update_views(con)


class TestDataWarehouseWorkflow:
    """Tests for complete data warehouse workflow."""

    @requires_warehouse
    def test_full_warehouse_workflow_with_mock_data(self):
        """Test complete workflow: init, load staging, process to analytics."""
        from mpcaHydro.data_manager_functions import init_warehouse
        from mpcaHydro import warehouse
        
        # Create mock WISKI-style data
        mock_wiski_staging = pd.DataFrame({
            'station_no': ['H001', 'H001', 'H002'],
            'Timestamp': ['2020-01-01 00:00', '2020-01-01 01:00', '2020-01-01 00:00'],
            'Value': [100.0, 105.0, 200.0],
            'Quality Code': [1, 1, 1],
            'ts_unitsymbol': ['cfs', 'cfs', 'cfs'],
            'parametertype_id': ['11500', '11500', '11500'],
            'parametertype_name': ['Discharge', 'Discharge', 'Discharge'],
        })
        
        mock_wiski_analytics = pd.DataFrame({
            'station_id': ['H001', 'H001', 'H002'],
            'datetime': pd.to_datetime(['2020-01-01 00:00', '2020-01-01 01:00', '2020-01-01 00:00']),
            'value': [100.0, 105.0, 200.0],
            'constituent': ['Q', 'Q', 'Q'],
            'unit': ['cfs', 'cfs', 'cfs'],
            'station_origin': ['wiski', 'wiski', 'wiski'],
        })
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            
            # Step 1: Initialize warehouse
            init_warehouse(db_path, reset=True)
            
            with warehouse.connect(db_path.as_posix(), read_only=False) as con:
                # Step 2: Load staging data
                warehouse.load_df_to_table(con, mock_wiski_staging, 'staging.wiski')
                
                # Verify staging data
                staging_count = con.execute("SELECT COUNT(*) FROM staging.wiski").fetchone()[0]
                assert staging_count == 3
                
                # Step 3: Load analytics data
                warehouse.load_df_to_table(con, mock_wiski_analytics, 'analytics.wiski')
                
                # Verify analytics data
                analytics_count = con.execute("SELECT COUNT(*) FROM analytics.wiski").fetchone()[0]
                assert analytics_count == 3
                
                # Step 4: Update views
                warehouse.update_views(con)


class TestDropStationData:
    """Tests for dropping station data."""

    @requires_warehouse
    def test_drop_wiski_station_data(self):
        """Test dropping WISKI station data."""
        from mpcaHydro.data_manager_functions import init_warehouse, drop_wiski_station_data
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            init_warehouse(db_path, reset=True)
            
            # Add test data
            staging_data = pd.DataFrame({
                'station_no': ['H001', 'H001', 'H002'],
                'Value': [1, 2, 3]
            })
            analytics_data = pd.DataFrame({
                'station_id': ['H001', 'H001', 'H002'],
                'value': [1.0, 2.0, 3.0]
            })
            
            with warehouse.connect(db_path.as_posix(), read_only=False) as con:
                warehouse.load_df_to_table(con, staging_data, 'staging.wiski')
                warehouse.load_df_to_table(con, analytics_data, 'analytics.wiski')
                
                # Verify initial data
                assert con.execute("SELECT COUNT(*) FROM staging.wiski WHERE station_no = 'H001'").fetchone()[0] == 2
                
                # Drop data for H001
                drop_wiski_station_data(con, ['H001'])
                
                # Verify H001 data is gone
                assert con.execute("SELECT COUNT(*) FROM staging.wiski WHERE station_no = 'H001'").fetchone()[0] == 0
                # H002 data should remain
                assert con.execute("SELECT COUNT(*) FROM staging.wiski WHERE station_no = 'H002'").fetchone()[0] == 1

    @requires_warehouse
    def test_drop_equis_station_data(self):
        """Test dropping EQuIS station data."""
        from mpcaHydro.data_manager_functions import init_warehouse, drop_equis_station_data
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            init_warehouse(db_path, reset=True)
            
            # Add test data
            staging_data = pd.DataFrame({
                'SYS_LOC_CODE': ['S001', 'S001', 'S002'],
                'VALUE': [1, 2, 3]
            })
            analytics_data = pd.DataFrame({
                'station_id': ['S001', 'S001', 'S002'],
                'value': [1.0, 2.0, 3.0]
            })
            
            with warehouse.connect(db_path.as_posix(), read_only=False) as con:
                warehouse.load_df_to_table(con, staging_data, 'staging.equis')
                warehouse.load_df_to_table(con, analytics_data, 'analytics.equis')
                
                # Verify initial data
                assert con.execute("SELECT COUNT(*) FROM staging.equis WHERE SYS_LOC_CODE = 'S001'").fetchone()[0] == 2
                
                # Drop data for S001
                drop_equis_station_data(con, ['S001'])
                
                # Verify S001 data is gone
                assert con.execute("SELECT COUNT(*) FROM staging.equis WHERE SYS_LOC_CODE = 'S001'").fetchone()[0] == 0
                # S002 data should remain
                assert con.execute("SELECT COUNT(*) FROM staging.equis WHERE SYS_LOC_CODE = 'S002'").fetchone()[0] == 1


class TestDataManagerWrapper:
    """Tests for DataManagerWrapper class."""

    def test_wrapper_initialization(self):
        """Test DataManagerWrapper can be initialized."""
        from mpcaHydro.data_manager_functions import DataManagerWrapper
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            wrapper = DataManagerWrapper(db_path)
            
            assert wrapper.db_path == db_path

    def test_wrapper_has_connection_method(self):
        """Test DataManagerWrapper has _connect method."""
        from mpcaHydro.data_manager_functions import DataManagerWrapper
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            wrapper = DataManagerWrapper(db_path)
            
            assert hasattr(wrapper, '_connect')
            assert callable(wrapper._connect)


if __name__ == '__main__':
    pytest.main([__file__, '-v'])



# -*- coding: utf-8 -*-
"""SQL file loader utilities for loading SQL from separate files."""

from pathlib import Path
from typing import Optional

# Path to the SQL folder
SQL_DIR = Path(__file__).parent / 'sql'


def load_sql(filename: str) -> str:
    """Load SQL content from a file in the sql folder."""
    filepath = SQL_DIR / filename
    if not filepath.exists():
        raise FileNotFoundError(f"SQL file not found: {filepath}")
    return filepath.read_text(encoding='utf-8')


def get_schemas_sql() -> str:
    """Load SQL for creating database schemas."""
    return load_sql('schemas.sql')


def get_staging_tables_sql() -> str:
    """Load SQL for creating staging tables."""
    return load_sql('staging_tables.sql')


def get_analytics_tables_sql() -> str:
    """Load SQL for creating analytics tables."""
    return load_sql('analytics_tables.sql')


def get_outlets_schema_sql() -> str:
    """Load SQL for creating outlets schema and tables."""
    return load_sql('outlets_schema.sql')


def get_views_analytics_sql() -> str:
    """Load SQL for creating analytics views."""
    return load_sql('views_analytics.sql')


def get_views_reports_sql() -> str:
    """Load SQL for creating reports views."""
    return load_sql('views_reports.sql')


def get_views_outlets_sql() -> str:
    """Load SQL for creating outlets views."""
    return load_sql('views_outlets.sql')


def get_calibration_schema_sql() -> str:
    """Load SQL for creating calibration schema (SQLite)."""
    return load_sql('calibration_schema.sql')



# -*- coding: utf-8 -*-
"""Integration tests for warehouse operations.

Note: Many tests require data files in src/mpcaHydro/data/ which may not be present.
These tests will be skipped if required files are missing.
"""

import tempfile
from pathlib import Path
import pytest
import pandas as pd


# Check if warehouse module can be imported (requires data files)
try:
    from mpcaHydro import warehouse
    WAREHOUSE_AVAILABLE = True
except Exception:
    WAREHOUSE_AVAILABLE = False
    warehouse = None

requires_warehouse = pytest.mark.skipif(
    not WAREHOUSE_AVAILABLE,
    reason="Warehouse module not available (missing data files)"
)


class TestWarehouseInitialization:
    """Tests for initializing the data warehouse."""

    @requires_warehouse
    def test_init_warehouse_creates_database(self):
        """Test that init_warehouse creates a database file."""
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            assert db_path.exists()

    @requires_warehouse
    def test_init_warehouse_creates_schemas(self):
        """Test that init_warehouse creates required schemas."""
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            with warehouse.connect(db_path.as_posix(), read_only=True) as con:
                schemas = con.execute(
                    "SELECT schema_name FROM information_schema.schemata"
                ).fetchall()
                schema_names = [s[0] for s in schemas]
                
                assert 'staging' in schema_names
                assert 'analytics' in schema_names
                assert 'mappings' in schema_names

    @requires_warehouse
    def test_init_warehouse_creates_staging_tables(self):
        """Test that init_warehouse creates staging tables."""
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            with warehouse.connect(db_path.as_posix(), read_only=True) as con:
                tables = con.execute(
                    "SELECT table_name FROM information_schema.tables WHERE table_schema = 'staging'"
                ).fetchall()
                table_names = [t[0] for t in tables]
                
                assert 'wiski' in table_names
                assert 'equis' in table_names

    @requires_warehouse
    def test_init_warehouse_creates_analytics_tables(self):
        """Test that init_warehouse creates analytics tables."""
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            with warehouse.connect(db_path.as_posix(), read_only=True) as con:
                tables = con.execute(
                    "SELECT table_name FROM information_schema.tables WHERE table_schema = 'analytics'"
                ).fetchall()
                table_names = [t[0] for t in tables]
                
                assert 'wiski' in table_names
                assert 'equis' in table_names

    @requires_warehouse
    def test_init_warehouse_reset_clears_data(self):
        """Test that reset=True clears existing database."""
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            
            # First initialization
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            # Insert some test data
            with warehouse.connect(db_path.as_posix(), read_only=False) as con:
                con.execute("INSERT INTO staging.wiski (station_no) VALUES ('TEST123')")
                count1 = con.execute("SELECT COUNT(*) FROM staging.wiski").fetchone()[0]
                assert count1 == 1
            
            # Reset and check data is cleared
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            with warehouse.connect(db_path.as_posix(), read_only=True) as con:
                count2 = con.execute("SELECT COUNT(*) FROM staging.wiski").fetchone()[0]
                assert count2 == 0


class TestWarehouseConnection:
    """Tests for warehouse connection management."""

    @requires_warehouse
    def test_connect_creates_parent_directory(self):
        """Test that connect creates parent directories if needed."""
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'subdir' / 'nested' / 'test.duckdb'
            
            # Initialize should create directories
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            assert db_path.exists()

    @requires_warehouse
    def test_connect_read_only_mode(self):
        """Test that read_only mode prevents writes."""
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            # Read-only connection should not allow INSERT
            with warehouse.connect(db_path.as_posix(), read_only=True) as con:
                with pytest.raises(Exception):
                    con.execute("INSERT INTO staging.wiski (station_no) VALUES ('TEST')")


class TestWarehouseDataLoading:
    """Tests for loading data into the warehouse."""

    @requires_warehouse
    def test_load_df_to_table(self):
        """Test loading a DataFrame into a table."""
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            df = pd.DataFrame({
                'station_no': ['A001', 'A002'],
                'value': [1.0, 2.0]
            })
            
            with warehouse.connect(db_path.as_posix(), read_only=False) as con:
                warehouse.load_df_to_table(con, df, 'staging.test_table')
                
                result = con.execute("SELECT * FROM staging.test_table").fetchdf()
                assert len(result) == 2
                assert 'station_no' in result.columns

    @requires_warehouse
    def test_load_df_to_staging(self):
        """Test loading a DataFrame to staging schema."""
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            df = pd.DataFrame({
                'id': [1, 2, 3],
                'name': ['a', 'b', 'c']
            })
            
            with warehouse.connect(db_path.as_posix(), read_only=False) as con:
                warehouse.load_df_to_staging(con, df, 'test_staging')
                
                result = con.execute("SELECT * FROM staging.test_staging").fetchdf()
                assert len(result) == 3


class TestWarehouseViews:
    """Tests for warehouse view operations."""

    @requires_warehouse
    def test_update_views_succeeds(self):
        """Test that update_views runs without error."""
        from mpcaHydro import warehouse
        
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            warehouse.init_db(db_path.as_posix(), reset=True)
            
            with warehouse.connect(db_path.as_posix(), read_only=False) as con:
                # Should not raise
                warehouse.update_views(con)


if __name__ == '__main__':
    pytest.main([__file__, '-v'])



# -*- coding: utf-8 -*-
"""Procedural functions for data management operations."""

from pathlib import Path
from typing import List, Optional, Union

import duckdb
import pandas as pd

from mpcaHydro import equis, warehouse, wiski

AGG_DEFAULTS = {
    'cfs': 'mean',
    'mg/l': 'mean',
    'degf': 'mean',
    'lb': 'sum'
}

UNIT_DEFAULTS = {
    'Q': 'cfs',
    'QB': 'cfs',
    'TSS': 'mg/l',
    'TP': 'mg/l',
    'OP': 'mg/l',
    'TKN': 'mg/l',
    'N': 'mg/l',
    'WT': 'degf',
    'WL': 'ft'
}


def get_db_path(folderpath: Union[str, Path]) -> Path:
    """Get the database path for a given folder."""
    return Path(folderpath) / 'observations.duckdb'


def init_warehouse(db_path: Union[str, Path], reset: bool = False) -> Path:
    """Initialize the data warehouse database and return the db_path."""
    from mpcaHydro import warehouse
    db_path = Path(db_path)
    warehouse.init_db(db_path.as_posix(), reset)
    return db_path


def update_views(con: duckdb.DuckDBPyConnection) -> None:
    """Update all database views."""
    from mpcaHydro import warehouse
    warehouse.update_views(con)


def process_wiski_data(
    con: duckdb.DuckDBPyConnection,
    filter_qc_codes: bool = True,
    data_codes: Optional[List[int]] = None,
    baseflow_method: str = 'Boughton'
) -> None:
    """Process WISKI data from staging to analytics."""
    from mpcaHydro import wiski, warehouse
    df = con.execute("SELECT * FROM staging.wiski").df()
    df_transformed = wiski.transform(df, filter_qc_codes, data_codes, baseflow_method)
    warehouse.load_df_to_table(con, df_transformed, 'analytics.wiski')
    warehouse.update_views(con)


def process_equis_data(con: duckdb.DuckDBPyConnection) -> None:
    """Process EQuIS data from staging to analytics."""
    from mpcaHydro import equis, warehouse
    df = con.execute("SELECT * FROM staging.equis").df()
    df_transformed = equis.transform(df)
    warehouse.load_df_to_table(con, df_transformed, 'analytics.equis')
    warehouse.update_views(con)


def process_all_data(
    con: duckdb.DuckDBPyConnection,
    filter_qc_codes: bool = True,
    data_codes: Optional[List[int]] = None,
    baseflow_method: str = 'Boughton'
) -> None:
    """Process all data (WISKI and EQuIS) from staging to analytics."""
    process_wiski_data(con, filter_qc_codes, data_codes, baseflow_method)
    process_equis_data(con)

                        
def download_wiski_data(
    con: duckdb.DuckDBPyConnection,
    station_ids: List[str],
    start_year: int = 1996,
    end_year: int = 2030,
    filter_qc_codes: bool = True,
    data_codes: Optional[List[int]] = None,
    baseflow_method: str = 'Boughton',
    overwrite: bool = True
) -> None:
    """Download WISKI data for given station IDs and load into the warehouse."""    

    df = wiski.download(station_ids, start_year=start_year, end_year=end_year)
    if not df.empty:
        df_transformed = wiski.transform(df.copy(), filter_qc_codes, data_codes, baseflow_method)
        # Drop existing data for these stations if overwrite is True
        if overwrite:
            warehouse.drop_station_data(con, station_ids, 'wiski')
        warehouse.add_df_to_table(con, df, 'staging', 'wiski')
        warehouse.add_df_to_table(con, df_transformed, 'analytics', 'wiski')
        warehouse.update_views(con)
    else:
        print('No data necessary for HSPF calibration from wiski for:', station_ids)


def download_equis_data(
    con: duckdb.DuckDBPyConnection,
    station_ids: List[str],
    oracle_username: str,
    oracle_password: str,
    overwrite: bool = True
) -> None:
    """Download EQuIS data for given station IDs and load into the warehouse."""    
    with equis.connect(user=oracle_username, password=oracle_password) as oracle_conn:
        print('Connected to Oracle database.')
        df = equis.download(station_ids, connection=oracle_conn)
    if not df.empty:
        df_transformed = equis.transform(df.copy())
        # Drop existing data for these stations if overwrite is True
        if overwrite:
            warehouse.drop_station_data(con, station_ids, 'equis')
        warehouse.add_df_to_table(con, df, 'staging', 'equis')
        warehouse.add_df_to_table(con, df_transformed, 'analytics', 'equis')
        warehouse.update_views(con)
    else:
        print('No data necessary for HSPF calibration from equis for:', station_ids)
    

def get_outlets(con: duckdb.DuckDBPyConnection, model_name: str) -> pd.DataFrame:
    """Get outlet data for a given model."""
    query = '''
    SELECT *
    FROM outlets.station_reach_pairs
    WHERE repository_name = ?
    ORDER BY outlet_id'''
    return con.execute(query, [model_name]).fetch_df()


def get_station_ids(
    con: duckdb.DuckDBPyConnection,
    station_origin: Optional[str] = None
) -> List[str]:
    """Get list of station IDs, optionally filtered by origin."""
    if station_origin is None:
        query = '''
        SELECT DISTINCT station_id, station_origin
        FROM analytics.observations'''
        df = con.execute(query).fetch_df()
    else:
        query = '''
        SELECT DISTINCT station_id
        FROM analytics.observations
        WHERE station_origin = ?'''
        df = con.execute(query, [station_origin]).fetch_df()
    return df['station_id'].to_list()


def get_observation_data(
    con: duckdb.DuckDBPyConnection,
    station_ids: List[str],
    constituent: str,
    agg_period: Optional[str] = None
) -> pd.DataFrame:
    """Get observation data for given stations and constituent."""
    query = '''
    SELECT *
    FROM analytics.observations
    WHERE station_id IN ? AND constituent = ?'''
    df = con.execute(query, [station_ids, constituent]).fetch_df()

    unit = UNIT_DEFAULTS.get(constituent, 'mg/l')
    agg_func = AGG_DEFAULTS.get(unit, 'mean')

    df.set_index('datetime', inplace=True)
    df.attrs['unit'] = unit
    df.attrs['constituent'] = constituent

    if agg_period is not None:
        df = df[['value']].resample(agg_period).agg(agg_func)
        df.attrs['agg_period'] = agg_period

    df.rename(columns={'value': 'observed'}, inplace=True)
    return df.dropna(subset=['observed'])


def get_outlet_data(
    con: duckdb.DuckDBPyConnection,
    outlet_id: int,
    constituent: str,
    agg_period: str = 'D'
) -> pd.DataFrame:
    """Get outlet observation data with flow for a given outlet and constituent."""
    query = '''
    SELECT *
    FROM analytics.outlet_observations_with_flow
    WHERE outlet_id = ? AND constituent = ?'''
    df = con.execute(query, [outlet_id, constituent]).fetch_df()

    unit = UNIT_DEFAULTS.get(constituent, 'mg/l')
    agg_func = AGG_DEFAULTS.get(unit, 'mean')

    df.set_index('datetime', inplace=True)
    df.attrs['unit'] = unit
    df.attrs['constituent'] = constituent

    if agg_period is not None:
        df = df[['value', 'flow_value', 'baseflow_value']].resample(agg_period).agg(agg_func)
        df.attrs['agg_period'] = agg_period

    df.rename(columns={
        'value': 'observed',
        'flow_value': 'observed_flow',
        'baseflow_value': 'observed_baseflow'
    }, inplace=True)
    return df.dropna(subset=['observed'])


def get_station_data(
    con: duckdb.DuckDBPyConnection,
    station_id: str,
    station_origin: str
) -> pd.DataFrame:
    """Get all observation data for a specific station."""
    query = '''
    SELECT *
    FROM analytics.observations
    WHERE station_id = ? AND station_origin = ?'''
    return con.execute(query, [station_id, station_origin]).fetch_df()


def get_raw_data(
    con: duckdb.DuckDBPyConnection,
    station_id: str,
    station_origin: str
) -> pd.DataFrame:
    """Get raw staging data for a specific station."""
    if station_origin.lower() == 'equis':
        query = '''
        SELECT *
        FROM staging.equis_raw
        WHERE station_id = ?'''
    elif station_origin.lower() == 'wiski':
        query = '''
        SELECT *
        FROM staging.wiski_raw
        WHERE station_id = ?'''
    else:
        raise ValueError(f'Station origin {station_origin} not recognized.')
    return con.execute(query, [station_id]).fetch_df()


def get_constituent_summary(con: duckdb.DuckDBPyConnection) -> pd.DataFrame:
    """Get summary of constituents across all stations."""
    query = '''
    SELECT
      station_id,
      station_origin,
      constituent,
      COUNT(*) AS sample_count,
      year(MIN(datetime)) AS start_date,
      year(MAX(datetime)) AS end_date
    FROM
      analytics.observations
    GROUP BY
      constituent, station_id, station_origin
    ORDER BY
      sample_count'''
    return con.execute(query).fetch_df()


def export_station_to_csv(
    con: duckdb.DuckDBPyConnection,
    station_id: str,
    station_origin: str,
    output_path: Union[str, Path]
) -> None:
    """Export station data to a CSV file."""
    df = get_station_data(con, station_id, station_origin)
    df.to_csv(output_path, index=False)


def export_raw_to_csv(
    con: duckdb.DuckDBPyConnection,
    station_id: str,
    station_origin: str,
    output_path: Union[str, Path]
) -> None:
    """Export raw staging data to a CSV file."""
    df = get_raw_data(con, station_id, station_origin)
    df.to_csv(output_path, index=False)


def get_equis_template(con: duckdb.DuckDBPyConnection) -> pd.DataFrame:
    """Get an empty DataFrame with EQuIS staging table schema."""
    query = '''SELECT * FROM staging.equis LIMIT 0'''
    return con.execute(query).fetch_df()


def get_wiski_template(con: duckdb.DuckDBPyConnection) -> pd.DataFrame:
    """Get an empty DataFrame with WISKI staging table schema."""
    query = '''SELECT * FROM staging.wiski LIMIT 0'''
    return con.execute(query).fetch_df()


def outlet_summary(con: duckdb.DuckDBPyConnection):
    query = '''
    SELECT *,
    FROM 
        reports.outlet_constituent_summary
    ORDER BY
        outlet_id,
        constituent
    '''
    df = con.execute(query).fetch_df()
    return df
        

def wiski_qc_counts(con: duckdb.DuckDBPyConnection):
    query = '''
    SELECT *,
    FROM 
        reports.wiski_qc_count
    ORDER BY
        station_no,
        parametertype_name
    '''
    df = con.execute(query).fetch_df()
    return df

def station_summary(con: duckdb.DuckDBPyConnection,constituent: str = None):
    
    query = '''
    SELECT *,
    FROM 
        reports.constituent_summary
    ORDER BY
        station_id,
        station_origin,
        constituent
    '''
    df = con.execute(query).fetch_df()
    if constituent is not None:
        df = df[df['constituent'] == constituent]
    return df

def station_reach_pairs(con: duckdb.DuckDBPyConnection):
    query = '''
    SELECT *,
    FROM 
        reports.station_reach_pairs
    ORDER BY
        outlet_id,
        station_id
    '''
    df = con.execute(query).fetch_df()
    return df

class DataManagerWrapper:
    """Minimal wrapper class that calls procedural functions with context-managed connections."""
    
    def __init__(self, db_path: Union[str, Path], reset: bool = False):
        """Initialize wrapper with database path."""
        self.db_path = Path(db_path)
        if reset:
            self._init_warehouse(reset=True)
    
    def _init_warehouse(self, reset: bool = False) -> None:
        """Initialize the data warehouse database."""
        init_warehouse(self.db_path, reset)

    def _connect(self, read_only: bool = False) -> duckdb.DuckDBPyConnection:
        """Create a database connection."""
        return warehouse.connect(self.db_path.as_posix(), read_only=read_only)
    
    def update_views(self) -> None:
        """Update all database views."""
        with self._connect(read_only=False) as con:
            update_views(con)
    
    def wiski_qc_counts(self):
        with self._connect(read_only=True) as con:
            return wiski_qc_counts(con)
        
    def station_summary(self,constituent: str = None):
        with self._connect(read_only=True) as con:
            return station_summary(con,constituent)
        
    def station_reach_pairs(self):
        with self._connect(read_only=True) as con:
            return station_reach_pairs(con)
        
    def outlet_summary(self):
        with self._connect(read_only=True) as con:
            return outlet_summary(con)
    
    
    def process_wiski_data(
        self,
        filter_qc_codes: bool = True,
        data_codes: Optional[List[int]] = None,
        baseflow_method: str = 'Boughton'
    ) -> None:
        """Process WISKI data from staging to analytics."""
        with self._connect(read_only=False) as con:
            process_wiski_data(con, filter_qc_codes, data_codes, baseflow_method)
    
    def process_equis_data(self) -> None:
        """Process EQuIS data from staging to analytics."""
        with self._connect(read_only=False) as con:
            process_equis_data(con)
    
    def process_all_data(
        self,
        filter_qc_codes: bool = True,
        data_codes: Optional[List[int]] = None,
        baseflow_method: str = 'Boughton'
    ) -> None:
        """Process all data (WISKI and EQuIS) from staging to analytics."""
        with self._connect(read_only=False) as con:
            process_all_data(con, filter_qc_codes, data_codes, baseflow_method)
    
    def download_wiski_data(
        self,
        station_ids: List[str],
        start_year: int = 1996,
        end_year: int = 2030,
        filter_qc_codes: bool = True,
        data_codes: Optional[List[int]] = None,
        baseflow_method: str = 'Boughton',
        replace: bool = False
    ) -> None:
        """Download WISKI data for given station IDs and load into the warehouse."""
        with self._connect(read_only=False) as con:
            download_wiski_data(
                con, station_ids, start_year, end_year,
                filter_qc_codes, data_codes, baseflow_method, replace
            )
    
    def download_equis_data(
        self,
        station_ids: List[str],
        oracle_username: str,
        oracle_password: str,
        replace: bool = False
    ) -> None:
        """Download EQuIS data for given station IDs and load into the warehouse."""
        with self._connect(read_only=False) as con:
            download_equis_data(con, station_ids, oracle_username, oracle_password, replace)
    
    def get_outlets(self, model_name: str) -> pd.DataFrame:
        """Get outlet data for a given model."""
        with self._connect(read_only=True) as con:
            return get_outlets(con, model_name)
    
    def get_station_ids(self, station_origin: Optional[str] = None) -> List[str]:
        """Get list of station IDs, optionally filtered by origin."""
        with self._connect(read_only=True) as con:
            return get_station_ids(con, station_origin)
    
    def get_observation_data(
        self,
        station_ids: List[str],
        constituent: str,
        agg_period: Optional[str] = None
    ) -> pd.DataFrame:
        """Get observation data for given stations and constituent."""
        with self._connect(read_only=True) as con:
            return get_observation_data(con, station_ids, constituent, agg_period)
    
    def get_outlet_data(
        self,
        outlet_id: int,
        constituent: str,
        agg_period: str = 'D'
    ) -> pd.DataFrame:
        """Get outlet observation data with flow for a given outlet and constituent."""
        with self._connect(read_only=True) as con:
            return get_outlet_data(con, outlet_id, constituent, agg_period)
    
    def get_station_data(self, station_id: str, station_origin: str) -> pd.DataFrame:
        """Get all observation data for a specific station."""
        with self._connect(read_only=True) as con:
            return get_station_data(con, station_id, station_origin)
    
    def get_raw_data(self, station_id: str, station_origin: str) -> pd.DataFrame:
        """Get raw staging data for a specific station."""
        with self._connect(read_only=True) as con:
            return get_raw_data(con, station_id, station_origin)
    
    def get_constituent_summary(self) -> pd.DataFrame:
        """Get summary of constituents across all stations."""
        with self._connect(read_only=True) as con:
            return get_constituent_summary(con)
    
    def export_station_to_csv(
        self,
        station_id: str,
        station_origin: str,
        output_path: Union[str, Path]
    ) -> None:
        """Export station data to a CSV file."""
        with self._connect(read_only=True) as con:
            export_station_to_csv(con, station_id, station_origin, output_path)
    
    def export_raw_to_csv(
        self,
        station_id: str,
        station_origin: str,
        output_path: Union[str, Path]
    ) -> None:
        """Export raw staging data to a CSV file."""
        with self._connect(read_only=True) as con:
            export_raw_to_csv(con, station_id, station_origin, output_path)
    
    def get_equis_template(self) -> pd.DataFrame:
        """Get an empty DataFrame with EQuIS staging table schema."""
        with self._connect(read_only=True) as con:
            return get_equis_template(con)
    
    def get_wiski_template(self) -> pd.DataFrame:
        """Get an empty DataFrame with WISKI staging table schema."""
        with self._connect(read_only=True) as con:
            return get_wiski_template(con)



# -*- coding: utf-8 -*-
"""Tests for the data_manager_functions procedural module."""

import tempfile
from pathlib import Path
from unittest.mock import patch, MagicMock
import pytest
import pandas as pd


class TestConstants:
    """Tests for module constants."""

    def test_agg_defaults(self):
        """Test aggregation defaults are properly defined."""
        from mpcaHydro.data_manager_functions import AGG_DEFAULTS
        assert 'cfs' in AGG_DEFAULTS
        assert AGG_DEFAULTS['cfs'] == 'mean'
        assert AGG_DEFAULTS['lb'] == 'sum'

    def test_unit_defaults(self):
        """Test unit defaults are properly defined."""
        from mpcaHydro.data_manager_functions import UNIT_DEFAULTS
        assert 'Q' in UNIT_DEFAULTS
        assert UNIT_DEFAULTS['Q'] == 'cfs'
        assert UNIT_DEFAULTS['TSS'] == 'mg/l'
        assert UNIT_DEFAULTS['WT'] == 'degf'


class TestGetDbPath:
    """Tests for get_db_path function."""

    def test_get_db_path_string(self):
        """Test get_db_path with string input."""
        from mpcaHydro.data_manager_functions import get_db_path
        result = get_db_path('/some/folder')
        assert result == Path('/some/folder/observations.duckdb')

    def test_get_db_path_path(self):
        """Test get_db_path with Path input."""
        from mpcaHydro.data_manager_functions import get_db_path
        result = get_db_path(Path('/another/folder'))
        assert result == Path('/another/folder/observations.duckdb')


class TestValidation:
    """Tests for validation constants."""

    def test_valid_constituents_in_unit_defaults(self):
        """Test that key constituents have unit defaults defined."""
        from mpcaHydro.data_manager_functions import UNIT_DEFAULTS
        expected_constituents = ['Q', 'TSS', 'TP', 'OP', 'TKN', 'N', 'WT']
        for const in expected_constituents:
            assert const in UNIT_DEFAULTS, f"Constituent {const} missing from UNIT_DEFAULTS"

    def test_valid_units_in_agg_defaults(self):
        """Test that common units have aggregation defaults defined."""
        from mpcaHydro.data_manager_functions import AGG_DEFAULTS
        expected_units = ['mg/l', 'lb', 'cfs', 'degf']
        for unit in expected_units:
            assert unit in AGG_DEFAULTS, f"Unit {unit} missing from AGG_DEFAULTS"


class TestDataManagerFunctionsInterface:
    """Test the interface of procedural functions."""

    def test_get_db_path_returns_path(self):
        """Test get_db_path returns a Path object."""
        from mpcaHydro.data_manager_functions import get_db_path
        result = get_db_path('/test/path')
        assert isinstance(result, Path)
        assert result.name == 'observations.duckdb'

    def test_constants_are_dicts(self):
        """Test constants are dictionaries."""
        from mpcaHydro.data_manager_functions import AGG_DEFAULTS, UNIT_DEFAULTS
        assert isinstance(AGG_DEFAULTS, dict)
        assert isinstance(UNIT_DEFAULTS, dict)


class TestDropStationFunctions:
    """Tests for drop station data functions."""

    def test_drop_wiski_station_data_exists(self):
        """Test drop_wiski_station_data function exists and is callable."""
        from mpcaHydro.data_manager_functions import drop_wiski_station_data
        assert callable(drop_wiski_station_data)

    def test_drop_equis_station_data_exists(self):
        """Test drop_equis_station_data function exists and is callable."""
        from mpcaHydro.data_manager_functions import drop_equis_station_data
        assert callable(drop_equis_station_data)


class TestDownloadFunctionsReplaceParameter:
    """Tests for replace parameter in download functions."""

    def test_download_wiski_data_has_replace_param(self):
        """Test download_wiski_data accepts replace parameter."""
        import inspect
        from mpcaHydro.data_manager_functions import download_wiski_data
        sig = inspect.signature(download_wiski_data)
        assert 'replace' in sig.parameters
        assert sig.parameters['replace'].default is False

    def test_download_equis_data_has_replace_param(self):
        """Test download_equis_data accepts replace parameter."""
        import inspect
        from mpcaHydro.data_manager_functions import download_equis_data
        sig = inspect.signature(download_equis_data)
        assert 'replace' in sig.parameters
        assert sig.parameters['replace'].default is False


class TestDataManagerWrapper:
    """Tests for DataManagerWrapper class."""

    def test_wrapper_exists(self):
        """Test DataManagerWrapper class exists."""
        from mpcaHydro.data_manager_functions import DataManagerWrapper
        assert DataManagerWrapper is not None

    def test_wrapper_can_be_instantiated(self):
        """Test DataManagerWrapper can be instantiated with a path."""
        from mpcaHydro.data_manager_functions import DataManagerWrapper
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            wrapper = DataManagerWrapper(db_path)
            assert wrapper.db_path == db_path

    def test_wrapper_has_expected_methods(self):
        """Test DataManagerWrapper has all expected methods."""
        from mpcaHydro.data_manager_functions import DataManagerWrapper
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            wrapper = DataManagerWrapper(db_path)
            
            # Check core methods exist
            expected_methods = [
                'update_views',
                'process_wiski_data',
                'process_equis_data',
                'process_all_data',
                'download_wiski_data',
                'download_equis_data',
                'get_outlets',
                'get_station_ids',
                'get_observation_data',
                'get_outlet_data',
                'get_station_data',
                'get_raw_data',
                'get_constituent_summary',
                'export_station_to_csv',
                'export_raw_to_csv',
                'get_equis_template',
                'get_wiski_template',
            ]
            
            for method_name in expected_methods:
                assert hasattr(wrapper, method_name), f"Missing method: {method_name}"
                assert callable(getattr(wrapper, method_name)), f"Not callable: {method_name}"

    def test_wrapper_download_methods_have_replace_param(self):
        """Test wrapper download methods have replace parameter."""
        import inspect
        from mpcaHydro.data_manager_functions import DataManagerWrapper
        with tempfile.TemporaryDirectory() as tmpdir:
            db_path = Path(tmpdir) / 'test.duckdb'
            wrapper = DataManagerWrapper(db_path)
            
            wiski_sig = inspect.signature(wrapper.download_wiski_data)
            assert 'replace' in wiski_sig.parameters
            
            equis_sig = inspect.signature(wrapper.download_equis_data)
            assert 'replace' in equis_sig.parameters


class TestModuleExports:
    """Test that new functions are exported from data_manager_functions module."""

    def test_drop_functions_exported(self):
        """Test drop functions are exported from data_manager_functions."""
        from mpcaHydro.data_manager_functions import drop_wiski_station_data, drop_equis_station_data
        assert callable(drop_wiski_station_data)
        assert callable(drop_equis_station_data)

    def test_wrapper_exported(self):
        """Test DataManagerWrapper is exported from data_manager_functions."""
        from mpcaHydro.data_manager_functions import DataManagerWrapper
        assert DataManagerWrapper is not None


if __name__ == '__main__':
    pytest.main([__file__, '-v'])



# -*- coding: utf-8 -*-
"""Pytest configuration for integration tests."""

import pytest


def pytest_configure(config):
    """Register custom markers."""
    config.addinivalue_line(
        "markers", "integration: mark test as integration test (requires full dependencies)"
    )
    config.addinivalue_line(
        "markers", "network: mark test as requiring network access"
    )
    config.addinivalue_line(
        "markers", "credentials: mark test as requiring external credentials"
    )



from mpcaHydro import wiski
from mpcaHydro import pywisk
from mpcaHydro import outlets

station_ids = ['H67014001']


df_info = wiski.info(station_ids,'Q')
start_year = int(df_info['from'].str[0:4].iloc[0])
wiski.discharge(station_ids,
                start_year = start_year, 
                end_year = start_year + 5)

model_name = 'CrowWing'
station_ids = outlets.mapped_wiski_stations(model_name)
df_info = wiski.info(station_ids,'Q')
start_year = df_info['from'].str[0:4].apply(lambda x:int(x)).min()
df = wiski.discharge(station_ids,
                start_year = start_year, 
                end_year = start_year + 2)

int(df_info['from'].str[0:4])
station_ids = ['H67009001',
 'H67012001',
 'H67013001',
 'H67014001',
 'H67016001',
 'H67014002',
 'E67013001',
 'W67036001',
 'W67004001']